/* Name: main.c
 
  $Id$

  based on the works found in

  DS1820 to USB connector http://www.poempelfox.de/ds1820tousb/
	Author: Michael Meier

  Little-wire http://littlewire.cc/
	Author: Ihsan Kehribar

  v-usb framework http://www.obdev.at/vusb/
	 Project: Thermostat based on AVR USB driver
	 Author: Christian Starkjohann
    
  usbtiny isp http://www.xs4all.nl/~dicks/avr/usbtiny/
  	Dick Streefland
  
  please observe licensing term from the above projects

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

 */

#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>

// Power reduction
#include <avr/power.h>

#include "usbdrv.h"
#include "oddebug.h"
#define sbi(register,bit) (register|=(1<<bit))
#define cbi(register,bit) (register&=~(1<<bit))

#define	PORT		PORTB
#define	DDR		DDRB
#define	PIN		PINB

#if defined(__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
	#define MASKB	3
	#define MASK	0x07
#else
	#define MASKB	7
	#define MASK	0xff
#endif

/* ------------------------------------------------------------------------- */
/* ------------------------ interface to USB driver ------------------------ */
/* ------------------------------------------------------------------------- */

uchar	usbFunctionSetup(uchar data[8])
{
// ----------------------------------------------------------------------
// Handle a non-standard SETUP packet.
// ----------------------------------------------------------------------
	uchar	req;

	// Generic requests
	req = data[1];
	if	( req == 0 ) // ECHO
	{
		data[1]=0x21;
		usbMsgPtr = data;
		return 8;
	}

#if defined(IOA)
	// IO support for port
	if( req == 1 ) // read PINB
	{
		data[0]=PINB & MASK;
		usbMsgPtr = data;
		return 1;
	}
	if( req == 2 ) // write PORTB
	{
		PORTB=(PORTB & ~MASK) | (data[2] & MASK);
		return 0;
	}
	if( req == 3 ) // write DDRB
	{
		DDRB=(DDRB & ~MASK) | (data[2] & MASK);
		return 0;
	}
#endif
#if defined(IOB)
	// IO support for pin
	if( req == 4 ) // set PORTBx
	{
		PORTB|=(1<<(data[2] & MASKB));
		return 0;
	}
	if( req == 5 ) // unset PORTBx
	{
		PORTB&=~(1<<(data[2] & MASKB));
		return 0;
	}
	if( req == 6 ) // out PORTBx
	{
		DDRB|=(1<<(data[2] & MASKB));
		return 0;
	}
	if( req == 6 ) // out PORTBx
	{
		DDRB&=~(1<<(data[2] & MASKB));
		return 0;
	}
#endif

#if defined(IADC)
	// ADC support
	if( req == 15 ) // READ ADC
	{
		unsigned char temp=DDR;
		if(data[2]<2) // Read ADC from SCK pin
		{
			ADMUX=data[2];
			DDR &= ~(1<<2);
		}
		else if(data[2]==2) // Read ADC from internal Temperature sensor
		{
			ADMUX=0b10001111;
		}
		ADCSRA|=(1<<ADSC);
		while(ADCSRA & (1<<ADSC));
		data[0]=(unsigned char)ADCL;
		data[1]=(unsigned char)ADCH;
		usbMsgPtr = data;
		DDR=temp;		
		return 2;
	}
#endif
	if( req == 253 ) // read FW config
	{
		data[0]=0x00;
#if defined(IOA)
		data[0]|=0x01;
#endif
#if defined(IOB)
		data[0]|=0x02;
#endif
#if defined(IADC)
		data[0]|=0x04;
#endif
#if defined(I2C)
		data[0]|=0x08;
#endif
#if defined(SWIRE)
		data[0]|=0x10;
#endif
#if defined(FWIRE)
                data[0]|=0x10;
#endif
#if defined(SPI)
                data[0]|=0x40;
#endif
		usbMsgPtr = data;
		return 1;
	}
	if( req == 254 ) // read CPU ID
	{
		data[0]=SIGNATURE_0;
		data[1]=SIGNATURE_1;
		data[2]=SIGNATURE_2;
		usbMsgPtr = data;
		return 3;
	}
	return 0;
}


#if defined(IPLLOSC) && ( defined(__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__))
/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

/* Calibrate the RC oscillator to 8.25 MHz. The core clock of 16.5 MHz is
 * derived from the 66 MHz peripheral clock by dividing. Our timing reference
 * is the Start Of Frame signal (a single SE0 bit) available immediately after
 * a USB RESET. We first do a binary search for the OSCCAL value and then
 * optimize this value with a neighboorhod search.
 * This algorithm may also be used to calibrate the RC oscillator directly to
 * 12 MHz (no PLL involved, can therefore be used on almost ALL AVRs), but this
 * is wide outside the spec for the OSCCAL value and the required precision for
 * the 12 MHz clock! Use the RC oscillator calibrated to 12 MHz for
 * experimental purposes only!
 */
static void calibrateOscillator(void)
{
uchar       step = 128;
uchar       trialValue = 0, optimumValue;
int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue; 
}

void    usbEventResetReady(void)
{
    calibrateOscillator();
    eeprom_write_byte(0, OSCCAL);   /* store the calibrated value in EEPROM */
}
#endif
/* ------------------------------------------------------------------------- */
/* --------------------------------- main ---------------------------------- */
/* ------------------------------------------------------------------------- */

int main(void) {
	uchar   i;
#if defined(IPLLOSC) && ( defined(__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__))
	uchar   calibrationValue;
#endif
#if defined(IPLLOSC) && ( defined(__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__))
    calibrationValue = eeprom_read_byte(0); /* calibration value from last time */
    if(calibrationValue != 0xff){
        OSCCAL = calibrationValue;
    }
#endif
    odDebugInit();
    usbDeviceDisconnect();
    for(i=0;i<20;i++){  /* 300 ms disconnect */
        _delay_ms(15);
    }
    usbDeviceConnect();

    wdt_enable(WDTO_1S);

    usbInit();
    sei();
#if defined(IADC)	
	// ADC init	
	ADMUX|=(0<<MUX3)|(0<<MUX2)|(0<<MUX1)|(1<<MUX0);
	ADMUX|=(0<<REFS2)|(0<<REFS1)|(0<<REFS0) |(0<<ADLAR);
	ADCSRA|= (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);
	ADCSRA|=(1<<ADEN);
	ADCSRA|=(1<<ADSC);
	MCUCR|=(1<<PUD);
	while(ADCSRA & (1<<ADSC));
#endif

// disable unused devices
#if !defined(IADC) && defined(power_adc_disable)
	power_adc_disable();
#endif
#if !defined(PWM) && defined(power_timer0_disable)
	power_timer0_disable();
#endif
#if !defined(PWM) && defined(power_timer1_disable)
	power_timer1_disable();
#endif
    for(;;)
	{   
        wdt_reset();
        usbPoll();
    }
    return 0;
}
